// SPDX-License-Identifier: GPL-2.0+
/*******************************************************************************
 * QtMips - MIPS 32-bit Architecture Subset Simulator
 *
 * Implemented to support following courses:
 *
 *   B35APO - Computer Architectures
 *   https://cw.fel.cvut.cz/wiki/courses/b35apo
 *
 *   B4M35PAP - Advanced Computer Architectures
 *   https://cw.fel.cvut.cz/wiki/courses/b4m35pap/start
 *
 * Copyright (c) 2017-2019 Karel Koci<cynerd@email.cz>
 * Copyright (c) 2019      Pavel Pisa <pisa@cmp.felk.cvut.cz>
 *
 * Faculty of Electrical Engineering (http://www.fel.cvut.cz)
 * Czech Technical University        (http://www.cvut.cz/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 ******************************************************************************/

#ifndef COREVIEW_REGISTERS_H
#define COREVIEW_REGISTERS_H

#include "connection.h"

#include <QGraphicsObject>
#include <QGraphicsSimpleTextItem>
#include <QPainter>

namespace coreview {

class Registers : public QGraphicsObject {
    Q_OBJECT
public:
    Registers();
    ~Registers() override;

    QRectF boundingRect() const override;
    void paint(
        QPainter *painter,
        const QStyleOptionGraphicsItem *option,
        QWidget *widget) override;

    void setPos(qreal x, qreal y);
    const Connector *connector_read1() const;
    const Connector *connector_read1_reg() const;
    const Connector *connector_read2() const;
    const Connector *connector_read2_reg() const;
    const Connector *connector_write() const;
    const Connector *connector_write_reg() const;
    const Connector *connector_ctl_write() const;

signals:
    void open_registers();

protected:
    void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event) override;

private:
    Connector *con_read1, *con_read1_reg, *con_read2, *con_read2_reg;
    Connector *con_write, *con_write_reg, *con_ctl_write;

    QGraphicsSimpleTextItem name;
};

} // namespace coreview

#endif // COREVIEW_REGISTERS_H
